"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formTemplateToChanges = exports.convertToTemplateBidningFromDb = exports.convertFormTemplateBindingFromDb = exports.resolveOutputBindingValue = exports.deepResolve = void 0;
const handlebars_1 = __importDefault(require("@timwoods/handlebars"));
const deepResolve = (fullkey, object) => {
    const keys = fullkey.split('.');
    let current = Object.assign({}, object);
    for (const key of keys) {
        if (typeof current === 'object' && current) {
            current = current[key];
        }
    }
    return current;
};
exports.deepResolve = deepResolve;
const resolveOutputBindingValue = (binding, fullContext) => {
    switch (binding.type) {
        case 'CONTEXT':
            return exports.deepResolve(`${binding.value.data_source}.${binding.value.data_key}`, fullContext);
        case 'CONTEXT_TEMPLATE':
            return handlebars_1.default.compile(binding.value)(fullContext);
        case 'VALUE':
            return binding.value;
    }
};
exports.resolveOutputBindingValue = resolveOutputBindingValue;
const convertFormTemplateBindingFromDb = (formTemplate) => {
    const clean = {};
    for (const key of Object.keys(formTemplate)) {
        clean[key.replace(/,/g, '.')] = formTemplate[key];
    }
    return clean;
};
exports.convertFormTemplateBindingFromDb = convertFormTemplateBindingFromDb;
const convertToTemplateBidningFromDb = (formTemplate) => {
    const clean = {};
    for (const key of Object.keys(formTemplate)) {
        clean[key.replace(/\./g, ',')] = formTemplate[key];
    }
    return clean;
};
exports.convertToTemplateBidningFromDb = convertToTemplateBidningFromDb;
const formTemplateToChanges = (form_data, context, formTemplate) => {
    var _a, _b;
    const fullContext = Object.assign(Object.assign({}, context), { form_data });
    const changes = [];
    const form_output_bindings = exports.convertFormTemplateBindingFromDb(formTemplate.form_output_bindings);
    const entity = {
        module: {
            module_name: ((_a = context.entity) === null || _a === void 0 ? void 0 : _a.module_name) || formTemplate.linked_module_name,
            module_id: ((_b = context.entity) === null || _b === void 0 ? void 0 : _b.id) || 'create',
        },
        changes: {},
    };
    const creator = {
        module: {
            module_name: context.creator.module_name,
            module_id: context.creator.id,
        },
        changes: {},
    };
    const actor = {
        module: {
            module_name: context.actor.module_name,
            module_id: context.actor.id,
        },
        changes: {},
    };
    for (const option of Object.keys(form_output_bindings)) {
        const binding = form_output_bindings[option];
        const value = exports.resolveOutputBindingValue(binding, fullContext);
        const keys = option.split('.');
        const target = keys.shift();
        const subkey = keys.join('.');
        switch (target) {
            case 'creator':
                creator.changes[subkey] = value;
                break;
            case 'entity':
                entity.changes[subkey] = value;
                break;
            case 'actor':
                actor.changes[subkey] = value;
                break;
        }
    }
    changes.push(entity);
    if (Object.keys(actor.changes).length !== 0) {
        changes.push(actor);
    }
    if (Object.keys(creator.changes).length !== 0) {
        changes.push(creator);
    }
    return changes;
};
exports.formTemplateToChanges = formTemplateToChanges;
